/// -----------------------------------------------------------------------------------------------------------
/// Module      :  SalesForceWCFAdapterOutboundHandler.cs
/// Description :  This class is used for sending data to the target system
/// -----------------------------------------------------------------------------------------------------------

#region Using Directives
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.ServiceModel.Channels;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.Xml.Linq;

using Microsoft.ServiceModel.Channels.Common;
#endregion

namespace WCFLOBTalk.SalesForceWCF
{
    public class SalesForceWCFAdapterOutboundHandler : SalesForceWCFAdapterHandlerBase, IOutboundHandler
    {
        /// <summary>
        /// Initializes a new instance of the SalesForceWCFAdapterOutboundHandler class
        /// </summary>
        public SalesForceWCFAdapterOutboundHandler(SalesForceWCFAdapterConnection connection
            , MetadataLookup metadataLookup)
            : base(connection, metadataLookup)
        {
        }

        #region IOutboundHandler Members

        /// <summary>
        /// Executes the request message on the target system and returns a response message.
        /// If there isnt a response, this method should return null
        /// </summary>
        public Message Execute(Message message, TimeSpan timeout)
        {
            // Connect to SalesForce
            string uri = Connection.ConnectionFactory.ConnectionProperties.Uri.ToString();
            string username = Connection.ConnectionFactory.Credentials.UserName.UserName;
            string password = Connection.ConnectionFactory.Credentials.UserName.Password;
            string securityToken = Connection.ConnectionFactory.Adapter.SecurityToken;
            string version = Connection.ConnectionFactory.Adapter.Version;

           SFService.SforceService s = SalesForceWCF.Helpers.SFHelper.Login(uri, username, password, securityToken, false, version); 

            // get the message's target method name.
            
            
            
            // Call 



            // create a message body for the response using the results converters
           StringBuilder outputString = new StringBuilder();
           XmlWriterSettings settings = new XmlWriterSettings();
           settings.OmitXmlDeclaration = true;
           // Create response message
           XmlWriter replywriter = XmlWriter.Create(outputString, settings);
           

            
           // replywriter.WriteStartElement("EchoStringResponse", EchoAdapter.SERVICENAMESPACE);
           //replywriter.WriteElementString("EchoStringResult", outputValue);
          // replywriter.WriteEndElement();
          // replywriter.Close();

           // Write a trace entry
          // WCFLOBTalk.SalesForceWCF.SalesForceWCFAdapterUtilities.Trace.Trace(TraceEventType.Verbose, "Hello", outputValue);

           XmlReader replyReader = XmlReader.Create(new StringReader(outputString.ToString()));
           // Output Message Format
           // <EchoStringResponse><EchoStringResult>{Name}</EchoStringResult></EchoStringResponse>
           // create output message
           return Message.CreateMessage(
                   MessageVersion.Default,
                   "Echo/EchoString/response",
                   replyReader);



        }

        #endregion IOutboundHandler Members
    }
}
