/// -----------------------------------------------------------------------------------------------------------
/// Module      :  SalesForceWCFAdapterMetadataResolverHandler.cs
/// Description :  This class is used for performing a connection-based retrieval of metadata from the target system.
/// ----------------------------------------------------------------------------------------------------------- 

#region Using Directives
using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.ServiceModel.Channels.Common;
#endregion

namespace WCFLOBTalk.SalesForceWCF
{
    public class SalesForceWCFAdapterMetadataResolverHandler : SalesForceWCFAdapterHandlerBase, IMetadataResolverHandler
    {
        /// <summary>
        /// Initializes a new instance of the SalesForceWCFAdapterMetadataResolverHandler class
        /// </summary>
        public SalesForceWCFAdapterMetadataResolverHandler(SalesForceWCFAdapterConnection connection
            , MetadataLookup metadataLookup)
            : base(connection, metadataLookup)
        {
        }

        #region IMetadataResolver Members

        /// <summary>
        /// Returns a value indicating whether the specified operation metadata is valid
        /// The DateTime field is provided to indicate when this specific operation metadata was last retrieved from the source system.
        /// The method should complete within the specified timespan or throw a timeout exception.
        /// </summary>
        public bool IsOperationMetadataValid(string operationId, DateTime lastUpdatedTimestamp, TimeSpan timeout)
        {
            //
            //TODO: Check the validity of the operation metadata on the target system.
            //      Examples include identifying any relevant changes in the target system, expiration of validity time interval etc.
            //

            return true;
        
        }

        /// <summary>
        /// Returns a value indicating whether the specified type metadata is valid.
        /// The DateTime field is provided to indicate when this specific type metadata was last retrieved from the source system.
        /// The method should complete within the specified timespan or throw a timeout exception.
        /// </summary>
        public bool IsTypeMetadataValid(string typeId, DateTime lastUpdatedTimestamp, TimeSpan timeout)
        {
            //
            //TODO: Check the validity of the type metadata on the target system.
            //      Metadata validity might constitute things like changes, expiry and etc.

            return true;
        
        }

        /// <summary>
        /// Returns an OperationMetadata object resolved from absolute name of the operation metadata object.
        /// The method should complete within the specified time span or throw a timeout exception.
        /// </summary>
        public OperationMetadata ResolveOperationMetadata(string operationId, TimeSpan timeout, out TypeMetadataCollection extraTypeMetadataResolved)
        {
            extraTypeMetadataResolved = null;

            //
            //TODO: Retrieve operation metadata from the target system and present it 
            //      as implementation of the OperationMetadata base class.
            //      Operation metadata represents operation signature, return type and etc.
            //

            ParameterizedOperationMetadata om = new ParameterizedOperationMetadata(operationId, operationId);
            // set this if you want this operation to belong to this interface name
            // in the generated proxy, the interface name will be EchoService
            // and the client implementation name will be EchoServiceClient.
            om.OperationGroup = "SalesForceWCFService";
            // set the operation namespace to be same as service namespace
            om.OperationNamespace = SalesForceWCFAdapter.SERVICENAMESPACE;   
 
            // Set other properties based on method or other details

            // Gotta remove the "/"
            string[] methodParts = operationId.Split(Convert.ToChar("/"));
            om.OriginalName = methodParts[2];
            om.DisplayName = String.Format("{0}.{1}", methodParts[1], methodParts[2]);

            return om;


        
        }

        /// <summary>
        /// Returns a TypeMetadata object resolved from the absolute name of the type metadata object.
        /// The method should complete within the specified time span or throw a timeout exception.
        /// </summary>
        public TypeMetadata ResolveTypeMetadata(string typeId, TimeSpan timeout, out TypeMetadataCollection extraTypeMetadataResolved)
        {
            extraTypeMetadataResolved = null;

            //
            //TODO: Retrieve type metadata from the target system and present it 
            //      as an implementation of the OperationMetadata base class.
            //      Type metadata represents target system supported types 
            //      as passed and returned through operation parameters.
            //
            throw new NotImplementedException("The method or operation is not implemented.");
        }

        #endregion IMetadataResolver Members
    }
}
